import demands

from tests.test_integration.helpers import create_site, create_user
from tests.test_integration.test_case import YolaServiceTestCase


class TestYolaUser(YolaServiceTestCase):
    """Yola: User resource"""

    @classmethod
    def setUpClass(cls):
        super(TestYolaUser, cls).setUpClass()
        cls.user = cls._create_user()
        cls.user_id = cls.user['id']

    @classmethod
    def _create_user(cls, **custom_attrs):
        custom_attrs.setdefault('name', 'John')
        return create_user(cls.service, **custom_attrs)

    def test_create_user(self):
        self.assertEqual(self.user['name'], 'John')

    def test_update_user(self):
        user = self._create_user(name='Original Name')
        user = self.service.update_user(user['id'], name='New Name')
        self.assertEqual(user['name'], 'New Name')

    def test_get_user(self):
        user = self.service.get_user(self.user_id)
        self.assertEqual(user['name'], 'John')

    def test_list_users(self):
        response = self.service.list_users(page_size=1)
        self.assertEqual(len(response['results']), 1)

    def test_request_filtered_user_list(self):
        response = self.service.list_users(id=self.user_id)
        self.assertEqual(len(response['results']), 1)
        self.assertEqual(response['results'][0]['id'], self.user_id)

    def test_delete_user(self):
        user = self._create_user()
        self.service.delete_user(user['id'])
        with self.assertRaises(demands.HTTPServiceError):
            self.service.get_user(user['id'])

    def test_suspend_and_resume_user(self):
        self.assertTrue(self.user['active'])

        self.service.suspend_user(self.user_id)
        user = self.service.get_user(self.user_id)
        self.assertFalse(user['active'])

        self.service.resume_user(self.user_id)
        user = self.service.get_user(self.user_id)
        self.assertTrue(user['active'])

    def test_get_sso_create_site_url(self):
        url = self.service.get_sso_create_site_url(self.user_id, 'example.com')
        self.assertTrue(url.startswith('http'))

    def test_get_sso_open_site_url(self):
        create_site(self.service, self.user_id)
        url = self.service.get_sso_open_site_url(self.user_id)
        self.assertTrue(url.startswith('http'))



from yolapy.services import Yola


class Subscription(object):

    """Construct a Subscription.

    The data in a Subscription instance does not persist until it has
    been saved.

    :param auto_renew: Bool
    :param billing_date: Datetime
    :param deprovision_date: Datetime
    :param expiry_date: Datetime
    :param id: Str, subscription's 32 char ID
    :param properties: Dict, subscription properties. If unset will be
        defaulted to the client's configured username:
        ``{"partner_id": "<CLIENT_USERNAME>"}``.
    :param sku: Int
    :param type: Str, subscription type
    :param user_id: Str, 32 character ID of the sub's user

    :return: Subscription
    :rtype: yolapy.models.Subscription

    """

    def __init__(
            self, auto_renew=False, billing_date=None, deprovision_date=None,
            expiry_date=None, id=None, properties=None, sku=None,
            start_date=None, status=None, term=None, type=None, user_id=None):
        self.client = Yola()

        self.auto_renew = auto_renew
        self.billing_date = billing_date
        self.deprovision_date = deprovision_date
        self.id = id
        self.properties = properties or {}
        self.properties.setdefault('partner_id', self.client.username)
        self.sku = sku
        self.type = type
        self.user_id = user_id

    @classmethod
    def list(cls, **kwargs):
        """Return a filtered list of Subscriptions.

        Useage::

           >>> from yolapy.models import Subscription
           >>> user_id = 'abcdef1234567890abcdef1234567890'
           >>> user_subs = Subscription.list(user_id=user_id)
           >>> print user_subs[0]
           <yolapy.models.subscription.Subscription object>
           >>> print user_subs[0].user_id
           u'abcdef1234567890abcdef1234567890'
        """
        response = Yola().list_subscriptions(**kwargs)
        return [cls(**sub) for sub in response['results']]



from yolapy.services import Yola


class User(object):

    """Yola User - a service model that uses the yola client for persistence.

    Example use:
    ```
    from yolapy.models import User as YolaUser

    yola_user = YolaUser(
        email='test@example.com',
        name='Jane',
        surname='Doe',
        partner_id='WL_PARTNER',
        preferences={'locale': 'en'})

    yola_user.save()
    ```
    """

    def __init__(self, active=False, deleted=None, email=None, id=None,
                 name=None, partner_id=None, preferences=None,
                 signup_date=None, surname=None):
        """Construct a Yola User.

        The data in a User instance does not persist until it has been saved.


        :param active: Bool, toggled ``False`` when user is deactivated
        :param deleted: Timestamp, date the user was deleted
        :param email: Str, user's email address
        :param id: Str, user's 32 character ID
        :param name: Str, user's first name
        :param partner_id: Str, yola partner responsible the user
        :param preferences: Dict, user's preferences
        :param signup_date: Timestamp, date the user signed up
        :param surname: Str, user's second name

        :return: :class:`User <User>` object
        :rtype: yolapy.users.models.User

        """
        self.client = Yola()

        self.partner_id = partner_id or self.client.username
        self.active = active
        self.deleted = deleted
        self.email = email
        self.id = id
        self.name = name
        self.preferences = preferences or {}
        self.signup_date = signup_date
        self.surname = surname

    def _save_create(self):
        user = self.client.create_user(
            email=self.email,
            name=self.name,
            partner_id=self.partner_id,
            preferences=self.preferences,
            surname=self.surname)
        self.update(**user)

    def _save_update(self):
        raise NotImplementedError()

    def save(self):
        """POST the user data back to the service."""
        if self.id:
            self._save_update()
        else:
            self._save_create()

    def update(self, **attributes):
        """Update all keyword arguments to update instance attributes."""
        for attr, val in attributes.items():
            setattr(self, attr, val)



from unittest import TestCase

from tests.test_integration import test_settings as settings
from yolapy.services import Yola


class YolaServiceTestCase(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.service = Yola(url=settings.url, auth=settings.auth)



from unittest import TestCase

from mock import patch

from yolapy.models import Subscription


class SubscriptionTestCase(TestCase):

    def setUp(self):
        self._mock_client()
        self._stub_list_subscriptions()

    def _mock_client(self):
        self.client_patcher = patch('yolapy.models.subscription.Yola')
        self.addCleanup(self.client_patcher.stop)
        self.client = self.client_patcher.start().return_value

    def _stub_list_subscriptions(self):
        self.client.list_subscriptions.return_value = {
            'results': [{
                'id': 'fake-sub-id-abcdef1234567890abcd',
            }],
        }


class SubscriptionList(SubscriptionTestCase):

    """Subscription.list()."""

    def test_fetches_subs_by_passing_arguments_to_client(self):
        Subscription.list(foo='bar', user_id='user-id')
        self.client.list_subscriptions.assert_called_with(
            foo='bar', user_id='user-id')

    def test_creates_subs_using_response(self):
        subs = Subscription.list()
        self.assertEqual(len(subs), 1)
        subscription = subs[0]
        self.assertEqual(subscription.id, 'fake-sub-id-abcdef1234567890abcd')


class SubscriptionProperties(SubscriptionTestCase):

    """Subscription.properties['partner_id']."""

    def test_uses_the_client_config_as_a_default(self):
        self.client.username = 'partner-id-foo123'
        sub = Subscription()
        self.assertEqual(sub.properties['partner_id'], 'partner-id-foo123')
